
#include <ngx_config.h>
#include <ngx_core.h>
#include <ngx_http.h>

#define MULTIPART_FORM_DATA_STRING              "multipart/form-data"
#define BOUNDARY_STRING                         "boundary="
#define CONTENT_DISPOSITION_STRING              "Content-Disposition:"
#define CONTENT_TYPE_STRING                     "Content-Type:"
#define FORM_DATA_STRING                        "form-data"
#define ATTACHMENT_STRING                       "attachment"
#define FILENAME_STRING                         "filename=\""
#define FIELDNAME_STRING                        "name=\""

#define DEFAULT_HEADER_ACCUMULATOR_SIZE         256
#define DEFAULT_OUTPUT_BUFFER_SIZE              2048

/*
 * State of upload context
 */
typedef enum {
	upload_state_boundary_seek,
	upload_state_after_boundary,
	upload_state_headers,
	upload_state_data,
	upload_state_finish
} upload_state_t;

/*
 * Upload configuration for specific location
 */
typedef struct {
    ngx_str_t         url;
    ngx_path_t        *store_path;
    ngx_uint_t        store_access;
} ngx_http_upload_loc_conf_t;

/*
 * Upload module context
 */
typedef struct ngx_http_upload_ctx_s {
	char                *boundary_data;
	char                *boundary_start;
	char                *boundary_pos;

	upload_state_t		upload_state;

	size_t              header_accumulator_size;
	char                *header_accumulator;
	char                *header_accumulator_pos;

    char                *field_name;
    char                *file_name;
    char                *content_type;

	size_t              output_buffer_size;
	u_char              *output_buffer;
	u_char              *output_buffer_pos;

    ngx_pool_t          *pool;

    unsigned int        first_part:1;
    unsigned int        discard_data:1;
    unsigned int        is_file:1;

    int (*start_part_f)(struct ngx_http_upload_ctx_s *upload_ctx);
    void (*finish_part_f)(struct ngx_http_upload_ctx_s *upload_ctx);
    void (*abort_part_f)(struct ngx_http_upload_ctx_s *upload_ctx);
	int (*flush_output_buffer_f)(struct ngx_http_upload_ctx_s *upload_ctx, u_char *buf, size_t len);

    ngx_http_request_t            *request;
    ngx_log_t                     *log;

    ngx_file_t                    output_file;
    ngx_chain_t                   *chain;
    ngx_chain_t                   *last;
} ngx_http_upload_ctx_t;

static ngx_int_t ngx_http_upload_handler(ngx_http_request_t *r);
static ngx_int_t ngx_http_upload_body_handler(ngx_http_request_t *r);

static void *ngx_http_upload_create_loc_conf(ngx_conf_t *cf);
static char *ngx_http_upload_merge_loc_conf(ngx_conf_t *cf,
    void *parent, void *child);
static char *ngx_http_upload_pass(ngx_conf_t *cf, ngx_command_t *cmd, void *conf);

static int ngx_http_upload_start(ngx_http_upload_ctx_t *u);
static void ngx_http_upload_finish(ngx_http_upload_ctx_t *u);
static void ngx_http_upload_abort(ngx_http_upload_ctx_t *u);

static int ngx_http_upload_flush_output_buffer(ngx_http_upload_ctx_t *u, u_char *buf, size_t len);

static void ngx_http_read_upload_client_request_body_handler(ngx_http_request_t *r);
static ngx_int_t ngx_http_do_read_upload_client_request_body(ngx_http_request_t *r);
static ngx_int_t ngx_http_process_request_body(ngx_http_request_t *r, ngx_chain_t *body);

ngx_int_t ngx_http_read_upload_client_request_body(ngx_http_request_t *r);

/*
 * upload_init_ctx
 *
 * Initialize upload context. Memory for upload context which is being passed
 * as upload_ctx parameter could be allocated anywhere and should not be freed
 * prior to upload_shutdown_ctx call.
 *
 * IMPORTANT:
 * 
 * After initialization the following routine SHOULD BE called:
 * 
 * upload_parse_content_type -- to assign part boundary 
 *
 * After initialization the following routines MAY BE called:
 *
 * upload_set_output_buffer_size -- to set size of output buffer
 * upload_set_header_accumulator_size -- to set size header accumulator
 * upload_set_application_ctx -- to assign custom application context
 * upload_set_start_upload_file_routine -- to assign custom start_upload_file routine
 * upload_set_finish_upload_file_routine -- to assign custom finish_upload_file routine
 * upload_set_abort_upload_file_routine -- to assign custom abort_upload_file routine
 * upload_set_flush_output_buffer_routine -- to assign custom flush_output_buffer routine
 *
 * Parameter:
 *     upload_ctx -- upload context which is being initialized
 * 
 */
void upload_init_ctx(ngx_http_upload_ctx_t *upload_ctx);

/*
 * upload_shutdown_ctx
 *
 * Shutdown upload context. Discard all remaining data and 
 * free all memory associated with upload context.
 *
 * Parameter:
 *     upload_ctx -- upload context which is being shut down
 * 
 */
void upload_shutdown_ctx(ngx_http_upload_ctx_t *upload_ctx);

/*
 * upload_start
 *
 * Starts multipart stream processing. Initializes internal buffers
 * and pointers
 *
 * Parameter:
 *     upload_ctx -- upload context which is being initialized
 * 
 * Return value:
 *               = 0 on success
 *               < 0 if some error is occured
 *
 */
int upload_start(ngx_http_upload_ctx_t *upload_ctx);

/*
 * upload_parse_content_type
 *
 * Parse and verify content type from HTTP header, extract boundary and
 * assign it to upload context
 * 
 * Parameters:
 *     upload_ctx -- upload context to populate
 *     content_type -- value of Content-Type header to parse
 *
 * Return value:
 *     0 on success
 *     -1 if some error occured
 */
int upload_parse_content_type(ngx_http_upload_ctx_t *upload_ctx, char *content_type);

/*
 * upload_set_output_buffer_size
 *
 * Sets size of output buffer for upload context. The less size of
 * output buffer is, the more ofter it is flushed, causing unnecessary  
 * overheads. However larger output buffer will consume more memory
 *
 * Parameters:
 *     upload_ctx -- upload context for which buffer size will be set
 *     output_buffer_size -- new size of output buffer 
 *
 */
void upload_set_output_buffer_size(ngx_http_upload_ctx_t *upload_ctx, size_t output_buffer_size);

/*
 * upload_get_output_buffer_size
 *
 * Retrieves size of output buffer for upload context.
 *
 * Parameters:
 *     upload_ctx -- upload context for which size of output buffer will be retrieved
 *
 */
size_t upload_get_output_buffer_size(ngx_http_upload_ctx_t *upload_ctx);

/*
 * upload_set_header_accumulator_size
 *
 * Sets size of header accumulator for upload context. This value
 * determines amount of memory used to accumulate part headers. Since
 * Content-Disposition header is accumulated in header accumulator,
 * this value influences directly on maximal size of the filename, which
 * can be processed by upload context. If the filename is too long to fit
 * into accumulator (including field name itself and field value),
 * stream will be discarded as malformed 
 *
 * Parameters:
 *     upload_ctx -- upload context for which allocate routine will be set
 *     header_accumulator_size -- new size of header accumulator 
 *
 */
void upload_set_header_accumulator_size(ngx_http_upload_ctx_t *upload_ctx, size_t header_accumulator_size);

/*
 * upload_process_buf
 *
 * Process buffer with multipart stream starting from start and terminating
 * by end, operating on upload_ctx. The header information is accumulated in
 * upload_ctx and could be retrieved using upload_get_file_content_type,
 * upload_get_file_name, upload_get_field_name functions. This call can issue
 * one or more calls to start_upload_file, finish_upload_file, abort_upload_file
 * and flush_output_buffer routines.
 *
 * Return value:
 *               > 0 if context is ready to process next buffer of data,
 *               = 0 if processing finished and remaining data could be discarded,
 *               < 0 if stream is malformed
 */
int upload_process_buf(ngx_http_upload_ctx_t *upload_ctx, u_char *start, u_char *end);

static ngx_command_t  ngx_http_upload_commands[] = { /* {{{ */

    /*
     * Enables uploads for location  
     */
    { ngx_string("upload_pass"),
      NGX_HTTP_LOC_CONF|NGX_CONF_TAKE1,
      ngx_http_upload_pass,
      NGX_HTTP_LOC_CONF_OFFSET,
      0,
      NULL },

    /*
     * Specifies base path of file store
     */
    { ngx_string("upload_store"),
      NGX_HTTP_MAIN_CONF|NGX_HTTP_SRV_CONF|NGX_HTTP_LOC_CONF|NGX_CONF_TAKE1,
      ngx_conf_set_path_slot,
      NGX_HTTP_LOC_CONF_OFFSET,
      offsetof(ngx_http_upload_loc_conf_t, store_path),
      NULL },

    /*
     * Specifies the access mode for files in store
     */
    { ngx_string("upload_store_access"),
      NGX_HTTP_MAIN_CONF|NGX_HTTP_SRV_CONF|NGX_HTTP_LOC_CONF|NGX_CONF_TAKE1,
      ngx_conf_set_access_slot,
      NGX_HTTP_LOC_CONF_OFFSET,
      offsetof(ngx_http_upload_loc_conf_t, store_access),
      NULL },

      ngx_null_command
}; /* }}} */

ngx_http_module_t  ngx_http_upload_module_ctx = { /* {{{ */
    NULL,                                  /* preconfiguration */
    NULL,                                  /* postconfiguration */

    NULL,                                  /* create main configuration */
    NULL,                                  /* init main configuration */

    NULL,                                  /* create server configuration */
    NULL,                                  /* merge server configuration */

    ngx_http_upload_create_loc_conf,          /* create location configuration */
    ngx_http_upload_merge_loc_conf            /* merge location configuration */
}; /* }}} */

ngx_module_t  ngx_http_upload_module = { /* {{{ */
    NGX_MODULE_V1,
    &ngx_http_upload_module_ctx,              /* module context */
    ngx_http_upload_commands,                 /* module directives */
    NGX_HTTP_MODULE,                       /* module type */
    NULL,                                  /* init master */
    NULL,                                  /* init module */
    NULL,                                  /* init process */
    NULL,                                  /* init thread */
    NULL,                                  /* exit thread */
    NULL,                                  /* exit process */
    NULL,                                  /* exit master */
    NGX_MODULE_V1_PADDING
}; /* }}} */

static ngx_int_t /* {{{ ngx_http_upload_handler */
ngx_http_upload_handler(ngx_http_request_t *r)
{
    ngx_http_upload_loc_conf_t  *ulcf;
    ngx_http_upload_ctx_t     *u;
    ngx_int_t                 rc;

    ulcf = ngx_http_get_module_loc_conf(r, ngx_http_upload_module);

    if (!(r->method & NGX_HTTP_POST)) {
        return NGX_DECLINED;
    }

    u = ngx_http_get_module_ctx(r, ngx_http_upload_module);

    if (u == NULL) {
        u = ngx_pcalloc(r->pool, sizeof(ngx_http_upload_ctx_t));
        if (u == NULL) {
            return NGX_HTTP_INTERNAL_SERVER_ERROR;
        }

        ngx_http_set_ctx(r, u, ngx_http_upload_module);
    }

    if(r->headers_in.content_type == 0) {
        // No content type header specified
        return NGX_HTTP_BAD_REQUEST;
    }

    u->request = r;
    u->log = r->connection->log;
    u->pool = r->pool;

    // Initialize upload context
    upload_init_ctx(u);

    {
        char content_type[r->headers_in.content_type->value.len+1];

        memcpy(content_type, r->headers_in.content_type->value.data, r->headers_in.content_type->value.len);
        content_type[r->headers_in.content_type->value.len] = '\0';

        if(upload_parse_content_type(u, content_type) < 0) {
            upload_shutdown_ctx(u);
            return NGX_HTTP_BAD_REQUEST;
        }
    }

	u->start_part_f = ngx_http_upload_start;
	u->finish_part_f = ngx_http_upload_finish;
	u->abort_part_f = ngx_http_upload_abort;
	u->flush_output_buffer_f = ngx_http_upload_flush_output_buffer;

    // Start upload
    if(upload_start(u) < 0)
        return NGX_HTTP_INTERNAL_SERVER_ERROR;

    rc = ngx_http_read_upload_client_request_body(r);

    if (rc >= NGX_HTTP_SPECIAL_RESPONSE) {
        return rc;
    }

    return NGX_DONE;
} /* }}} */

static ngx_int_t /* {{{ ngx_http_upload_output */
ngx_http_upload_output(ngx_http_request_t *r, void *data, ngx_int_t rc)
{
    if (rc == NGX_ERROR || r->connection->error) {
        return rc;
    }

    if (!r->header_sent) {
        if (ngx_http_set_content_type(r) == NGX_ERROR) {
            return NGX_ERROR;
        }

        if (ngx_http_send_header(r) == NGX_ERROR) {
            return NGX_ERROR;
        }
    }

    //ngx_http_clear_content_length(r->main);

    ngx_http_send_special(r, NGX_HTTP_LAST|NGX_HTTP_FLUSH);

    return rc;
} /* }}} */

static ngx_int_t ngx_http_upload_body_handler(ngx_http_request_t *r) { /* {{{ */
    ngx_http_upload_loc_conf_t  *ulcf = ngx_http_get_module_loc_conf(r, ngx_http_upload_module);
    ngx_http_upload_ctx_t       *ctx = ngx_http_get_module_ctx(r, ngx_http_upload_module);

    ngx_str_t                   args;
    ngx_uint_t                  flags;
    ngx_int_t                   rc;
    ngx_str_t                   *uri;
    ngx_http_post_subrequest_t  *psr;
    ngx_http_request_t          *sr;
    ngx_buf_t                      *b;
    ngx_chain_t                    *cl;

    b = ngx_create_temp_buf(r->pool, strlen(ctx->boundary_data) + 2);

    if (b == NULL) {
        ngx_http_finalize_request(r, NGX_HTTP_INTERNAL_SERVER_ERROR);
        return NGX_HTTP_INTERNAL_SERVER_ERROR;
    }

    cl = ngx_alloc_chain_link(r->pool);
    if (cl == NULL) {
        ngx_http_finalize_request(r, NGX_HTTP_INTERNAL_SERVER_ERROR);
        return NGX_HTTP_INTERNAL_SERVER_ERROR;
    }

    b->last_in_chain = 1;

    cl->buf = b;
    cl->next = NULL;
    
    if(ctx->chain == NULL) {
        ctx->chain = cl;
        ctx->last = cl;
    }else{
        ctx->last->next = cl;
        ctx->last = cl;
    }

    b->last = ngx_cpymem(b->last, ctx->boundary_data, strlen(ctx->boundary_data));

    *b->last++ = '-';
    *b->last++ = '-';

    uri = &ulcf->url;

    args.len = 0;
    args.data = NULL;
    flags = 0;

    if (ngx_http_parse_unsafe_uri(r, uri, &args, &flags) != NGX_OK) {
        ngx_http_finalize_request(r, NGX_HTTP_INTERNAL_SERVER_ERROR);
        return NGX_HTTP_INTERNAL_SERVER_ERROR;
    }

    psr = ngx_palloc(r->pool, sizeof(ngx_http_post_subrequest_t));
    if (psr == NULL) {
        ngx_http_finalize_request(r, NGX_HTTP_INTERNAL_SERVER_ERROR);
        return NGX_HTTP_INTERNAL_SERVER_ERROR;
    }

    r->request_body->bufs = ctx->chain;

    // Recalculate content length
    r->headers_in.content_length_n = 0;

    for(cl = ctx->chain ; cl ; cl = cl->next) {
        r->headers_in.content_length_n += (cl->buf->last - cl->buf->pos);
    }

    r->headers_in.content_length->value.data = ngx_palloc(r->pool, NGX_OFF_T_LEN);

    if (r->headers_in.content_length->value.data == NULL) {
        ngx_http_finalize_request(r, NGX_HTTP_INTERNAL_SERVER_ERROR);
        return NGX_HTTP_INTERNAL_SERVER_ERROR;
    }

    r->headers_in.content_length->value.len =
        ngx_sprintf(r->headers_in.content_length->value.data, "%O", r->headers_in.content_length_n)
            - r->headers_in.content_length->value.data;

    psr->handler = ngx_http_upload_output;
    psr->data = ctx->chain;

    rc = ngx_http_subrequest(r, uri, &args, &sr, psr, flags);

    if (rc == NGX_ERROR) {
        ngx_http_finalize_request(r, NGX_HTTP_INTERNAL_SERVER_ERROR);
        return NGX_HTTP_INTERNAL_SERVER_ERROR;
    }

    r->keepalive = 0;

    rc = ngx_http_send_header(r);

    return rc;
} /* }}} */

static int ngx_http_upload_start(ngx_http_upload_ctx_t *u) { /* {{{ */
    ngx_http_request_t        *r = u->request;
    ngx_http_upload_loc_conf_t  *ulcf = ngx_http_get_module_loc_conf(r, ngx_http_upload_module);

    ngx_buf_t   *b;
    ngx_chain_t *cl;
    ngx_int_t   len;
    ngx_file_t  *file = &u->output_file;
    ngx_path_t  *path = ulcf->store_path;
    ngx_str_t   boundary;
    uint32_t    n;

    if(u->is_file) {
        file->name.len = path->name.len + 1 + path->len + 10;

        file->name.data = ngx_palloc(u->pool, file->name.len + 1);

        if(file->name.data == NULL)
            return -1;

        ngx_memcpy(file->name.data, path->name.data, path->name.len);

        file->log = u->log;

        n = (uint32_t) ngx_next_temp_number(0);

        (void) ngx_sprintf(file->name.data + path->name.len + 1 + path->len,
                           "%010uD%Z", n);

        ngx_create_hashed_filename(path, file->name.data, file->name.len);

        ngx_log_debug1(NGX_LOG_DEBUG_CORE, file->log, 0,
                       "hashed path: %s", file->name.data);

        file->fd = ngx_open_tempfile(file->name.data, 1, ulcf->store_access);

        if (file->fd == NGX_INVALID_FILE) {
            ngx_log_error(NGX_LOG_ERR, u->log, ngx_errno,
                          "failed to create output file for \"%s\"", u->file_name);
            return -1;
        }

        file->offset = 0;
    }

    boundary.data = u->first_part ? (u_char*)u->boundary_data + 2 : (u_char*)u->boundary_data;
    boundary.len = strlen((char*)boundary.data);

    len = boundary.len;

    len += sizeof("\r\nContent-Disposition: form-data; name=\"") - 1;

    len += strlen(u->field_name);

    len += sizeof("\"\r\n\r\n") - 1;

    if(u->is_file)
        len += file->name.len + 1 + strlen(u->file_name);

    b = ngx_create_temp_buf(u->pool, len);

    if (b == NULL) {
        return -1;
    }

    cl = ngx_alloc_chain_link(u->pool);
    if (cl == NULL) {
        return -1;
    }

    b->last_in_chain = 0;

    cl->buf = b;
    cl->next = NULL;
    
    if(u->chain == NULL) {
        u->chain = cl;
        u->last = cl;
    }else{
        u->last->next = cl;
        u->last = cl;
    }

    b->last = ngx_cpymem(b->last, boundary.data, boundary.len);

    b->last = ngx_cpymem(b->last, "\r\nContent-Disposition: form-data; name=\"", sizeof("\r\nContent-Disposition: form-data; name=\"") - 1);

    b->last = ngx_cpymem(b->last, u->field_name, strlen(u->field_name));

    b->last = ngx_cpymem(b->last, "\"\r\n\r\n", sizeof("\"\r\n\r\n") - 1);

    if(u->is_file) {
        b->last = ngx_cpymem(b->last, file->name.data, file->name.len);

        *b->last++ = ' ';

        b->last = ngx_cpymem(b->last, u->file_name, strlen(u->file_name));

        ngx_log_error(NGX_LOG_INFO, u->log, 0
            , "starting uploading file \"%s\" (field \"%s\", content type \"%s\") to \"%s\""
            , u->file_name
            , u->field_name
            , u->content_type
            , file->name.data
            );
    }

    return 0;
} /* }}} */

static void ngx_http_upload_finish(ngx_http_upload_ctx_t *u) { /* {{{ */
    if(u->is_file) {
        ngx_close_file(u->output_file.fd);

        ngx_log_error(NGX_LOG_INFO, u->log, 0
            , "finished uploading file \"%s\" to \"%s\""
            , u->file_name
            , u->output_file.name.data
            );
    }
} /* }}} */

static void ngx_http_upload_abort(ngx_http_upload_ctx_t *u) { /* {{{ */
    if(u->is_file) {

        ngx_close_file(u->output_file.fd);

        if(ngx_delete_file(u->output_file.name.data) == NGX_FILE_ERROR) { 
            ngx_log_error(NGX_LOG_ERR, u->log, 0
                , "aborted uploading file \"%s\" to \"%s\", failed to remove destination file"
                , u->file_name
                , u->output_file.name.data);
        } else {
            ngx_log_error(NGX_LOG_ALERT, u->log, 0
                , "aborted uploading file \"%s\" to \"%s\", dest file removed"
                , u->file_name
                , u->output_file.name.data);
        }
    }
} /* }}} */

static int ngx_http_upload_flush_output_buffer(ngx_http_upload_ctx_t *u, u_char *buf, size_t len) { /* {{{ */
    ngx_buf_t                      *b;
    ngx_chain_t                    *cl;

    if(u->is_file) {
        if(ngx_write_file(&u->output_file, buf, len, u->output_file.offset) == NGX_ERROR)
            return -1;
        else
            return 0;
    }else{
        b = ngx_create_temp_buf(u->pool, len);

        if (b == NULL) {
            return -1;
        }

        cl = ngx_alloc_chain_link(u->pool);
        if (cl == NULL) {
            return -1;
        }

        b->last_in_chain = 0;

        cl->buf = b;
        cl->next = NULL;
        
        if(u->chain == NULL) {
            u->chain = cl;
            u->last = cl;
        }else{
            u->last->next = cl;
            u->last = cl;
        }

        b->last = ngx_cpymem(b->last, buf, len);

        return 0;
    }
} /* }}} */

static void * /* {{{ ngx_http_upload_create_loc_conf */
ngx_http_upload_create_loc_conf(ngx_conf_t *cf)
{
    ngx_http_upload_loc_conf_t  *conf;

    conf = ngx_pcalloc(cf->pool, sizeof(ngx_http_upload_loc_conf_t));
    if (conf == NULL) {
        return NGX_CONF_ERROR;
    }

    conf->store_access = 0600;

    return conf;
} /* }}} */

static char * /* {{{ ngx_http_upload_merge_loc_conf */
ngx_http_upload_merge_loc_conf(ngx_conf_t *cf, void *parent, void *child)
{
    ngx_http_upload_loc_conf_t  *prev = parent;
    ngx_http_upload_loc_conf_t  *conf = child;

    ngx_conf_merge_str_value(conf->url, prev->url, "");

    ngx_conf_merge_path_value(conf->store_path,
                              prev->store_path,
                              NGX_HTTP_PROXY_TEMP_PATH, 1, 2, 0,
                              ngx_garbage_collector_temp_handler, cf);

    ngx_conf_merge_uint_value(conf->store_access,
                              prev->store_access, 0600);

    return NGX_CONF_OK;
} /* }}} */

/*
 * Initialize upload module
 */
static char * /* {{{ ngx_http_upload_pass  */
ngx_http_upload_pass(ngx_conf_t *cf, ngx_command_t *cmd, void *conf)
{
    ngx_http_core_loc_conf_t    *clcf;
    ngx_http_upload_loc_conf_t  *ulcf = conf;

    ngx_str_t                   *value, *url;

    clcf = ngx_http_conf_get_module_loc_conf(cf, ngx_http_core_module);
    clcf->handler = ngx_http_upload_handler;

    value = cf->args->elts;

    url = &value[1];

    ulcf->url = *url;

    return NGX_CONF_OK;
} /* }}} */

ngx_int_t /* {{{ ngx_http_read_upload_client_request_body */
ngx_http_read_upload_client_request_body(ngx_http_request_t *r) {
    ssize_t                    size, preread;
    ngx_buf_t                 *b;
    ngx_chain_t               *cl, **next;
    ngx_http_request_body_t   *rb;
    ngx_http_core_loc_conf_t  *clcf;
    ngx_http_upload_ctx_t     *u = ngx_http_get_module_ctx(r, ngx_http_upload_module);

    if (r->request_body || r->discard_body) {
        return NGX_OK;
    }

    rb = ngx_pcalloc(r->pool, sizeof(ngx_http_request_body_t));
    if (rb == NULL) {
        upload_shutdown_ctx(u);
        return NGX_HTTP_INTERNAL_SERVER_ERROR;
    }

    r->request_body = rb;

    if (r->headers_in.content_length_n <= 0) {
        upload_shutdown_ctx(u);
        return NGX_HTTP_BAD_REQUEST;
    }

    /*
     * set by ngx_pcalloc():
     *
     *     rb->bufs = NULL;
     *     rb->buf = NULL;
     *     rb->rest = 0;
     */

    preread = r->header_in->last - r->header_in->pos;

    if (preread) {

        /* there is the pre-read part of the request body */

        ngx_log_debug1(NGX_LOG_DEBUG_HTTP, r->connection->log, 0,
                       "http client request body preread %uz", preread);

        b = ngx_calloc_buf(r->pool);
        if (b == NULL) {
            upload_shutdown_ctx(u);
            return NGX_HTTP_INTERNAL_SERVER_ERROR;
        }

        b->temporary = 1;
        b->start = r->header_in->pos;
        b->pos = r->header_in->pos;
        b->last = r->header_in->last;
        b->end = r->header_in->end;

        rb->bufs = ngx_alloc_chain_link(r->pool);
        if (rb->bufs == NULL) {
            upload_shutdown_ctx(u);
            return NGX_HTTP_INTERNAL_SERVER_ERROR;
        }

        rb->bufs->buf = b;
        rb->bufs->next = NULL;

        if (preread >= r->headers_in.content_length_n) {

            /* the whole request body was pre-read */

            r->header_in->pos += r->headers_in.content_length_n;
            r->request_length += r->headers_in.content_length_n;

            if (ngx_http_process_request_body(r, rb->bufs) != NGX_OK) {
                upload_shutdown_ctx(u);
                return NGX_HTTP_INTERNAL_SERVER_ERROR;
            }
            
            upload_shutdown_ctx(u);

            return ngx_http_upload_body_handler(r);
        }

        /*
         * to not consider the body as pipelined request in
         * ngx_http_set_keepalive()
         */
        r->header_in->pos = r->header_in->last;

        r->request_length += preread;

        rb->rest = r->headers_in.content_length_n - preread;

        if (rb->rest <= (size_t) (b->end - b->last)) {

            /* the whole request body may be placed in r->header_in */

            rb->buf = b;

            r->read_event_handler = ngx_http_read_upload_client_request_body_handler;

            return ngx_http_do_read_upload_client_request_body(r);
        }

        next = &rb->bufs->next;

    } else {
        b = NULL;
        rb->rest = r->headers_in.content_length_n;
        next = &rb->bufs;
    }

    clcf = ngx_http_get_module_loc_conf(r, ngx_http_core_module);

    size = clcf->client_body_buffer_size;
    size += size >> 2;

    if (rb->rest < (size_t) size) {
        size = rb->rest;

        if (r->request_body_in_single_buf) {
            size += preread;
        }

    } else {
        size = clcf->client_body_buffer_size;

        /* disable copying buffer for r->request_body_in_single_buf */
        b = NULL;
    }

    rb->buf = ngx_create_temp_buf(r->pool, size);
    if (rb->buf == NULL) {
        upload_shutdown_ctx(u);
        return NGX_HTTP_INTERNAL_SERVER_ERROR;
    }

    cl = ngx_alloc_chain_link(r->pool);
    if (cl == NULL) {
        upload_shutdown_ctx(u);
        return NGX_HTTP_INTERNAL_SERVER_ERROR;
    }

    cl->buf = rb->buf;
    cl->next = NULL;

    if (b && r->request_body_in_single_buf) {
        size = b->last - b->pos;
        ngx_memcpy(rb->buf->pos, b->pos, size);
        rb->buf->last += size;

        next = &rb->bufs;
    }

    *next = cl;

    rb->to_write = rb->bufs;

    r->read_event_handler = ngx_http_read_upload_client_request_body_handler;

    return ngx_http_do_read_upload_client_request_body(r);
} /* }}} */

static void /* {{{ */
ngx_http_read_upload_client_request_body_handler(ngx_http_request_t *r)
{
    ngx_int_t  rc;
    ngx_http_upload_ctx_t     *u = ngx_http_get_module_ctx(r, ngx_http_upload_module);

    if (r->connection->read->timedout) {
        r->connection->timedout = 1;
        upload_shutdown_ctx(u);
        ngx_http_finalize_request(r, NGX_HTTP_REQUEST_TIME_OUT);
        return;
    }

    rc = ngx_http_do_read_upload_client_request_body(r);

    if (rc >= NGX_HTTP_SPECIAL_RESPONSE) {
        upload_shutdown_ctx(u);
        ngx_http_finalize_request(r, rc);
    }
} /* }}} */

static ngx_int_t /* {{{ */
ngx_http_do_read_upload_client_request_body(ngx_http_request_t *r)
{
    size_t                     size;
    ssize_t                    n;
    ngx_connection_t          *c;
    ngx_http_request_body_t   *rb;
    ngx_http_core_loc_conf_t  *clcf;
    ngx_http_upload_ctx_t     *u = ngx_http_get_module_ctx(r, ngx_http_upload_module);

    c = r->connection;
    rb = r->request_body;

    ngx_log_debug0(NGX_LOG_DEBUG_HTTP, c->log, 0,
                   "http read client request body");

    for ( ;; ) {
        for ( ;; ) {
            if (rb->buf->last == rb->buf->end) {

                if (ngx_http_process_request_body(r, rb->to_write) != NGX_OK) {
                    return NGX_HTTP_INTERNAL_SERVER_ERROR;
                }

                rb->to_write = rb->bufs->next ? rb->bufs->next : rb->bufs;
                rb->buf->last = rb->buf->start;
            }

            size = rb->buf->end - rb->buf->last;

            if (size > rb->rest) {
                size = rb->rest;
            }

            n = c->recv(c, rb->buf->last, size);

            ngx_log_debug1(NGX_LOG_DEBUG_HTTP, c->log, 0,
                           "http client request body recv %z", n);

            if (n == NGX_AGAIN) {
                break;
            }

            if (n == 0) {
                ngx_log_error(NGX_LOG_INFO, c->log, 0,
                              "client closed prematurely connection");
            }

            if (n == 0 || n == NGX_ERROR) {
                c->error = 1;
                return NGX_HTTP_BAD_REQUEST;
            }

            rb->buf->last += n;
            rb->rest -= n;
            r->request_length += n;

            if (rb->rest == 0) {
                break;
            }

            if (rb->buf->last < rb->buf->end) {
                break;
            }
        }

        ngx_log_debug1(NGX_LOG_DEBUG_HTTP, c->log, 0,
                       "http client request body rest %uz", rb->rest);

        if (rb->rest == 0) {
            break;
        }

        if (!c->read->ready) {
            clcf = ngx_http_get_module_loc_conf(r, ngx_http_core_module);
            ngx_add_timer(c->read, clcf->client_body_timeout);

            if (ngx_handle_read_event(c->read, 0) == NGX_ERROR) {
                return NGX_HTTP_INTERNAL_SERVER_ERROR;
            }

            return NGX_AGAIN;
        }
    }

    if (c->read->timer_set) {
        ngx_del_timer(c->read);
    }

    if (ngx_http_process_request_body(r, rb->to_write) != NGX_OK) {
        return NGX_HTTP_INTERNAL_SERVER_ERROR;
    }

    upload_shutdown_ctx(u);

    return ngx_http_upload_body_handler(r);
} /* }}} */

static ngx_int_t /* {{{ */
ngx_http_process_request_body(ngx_http_request_t *r, ngx_chain_t *body)
{
    ngx_http_upload_ctx_t     *u = ngx_http_get_module_ctx(r, ngx_http_upload_module);

    // Feed all the buffers into multipart/form-data processor
    while(body) {
        if(upload_process_buf(u, body->buf->pos, body->buf->last) < 0)
            return NGX_ERROR;
        body = body->next;
    }

    return NGX_OK;
} /* }}} */

/*
 * Static helper functions
 */
static void upload_reset_header_accumulator(ngx_http_upload_ctx_t *upload_ctx) { /* {{{ */
	memset(upload_ctx->header_accumulator, 0, upload_ctx->header_accumulator_size);
	upload_ctx->header_accumulator_pos = upload_ctx->header_accumulator;
} /* }}} */

static int upload_parse_part_header(ngx_http_upload_ctx_t *upload_ctx, char *header) { /* {{{ */
    if(!strncasecmp(CONTENT_DISPOSITION_STRING, header, strlen(CONTENT_DISPOSITION_STRING))) {
        char *p = header + strlen(CONTENT_DISPOSITION_STRING);
        char *filename_start, *filename_end;
        char *fieldname_start, *fieldname_end;

        p += strspn(p, " ");
        
        if(strncasecmp(FORM_DATA_STRING, p, sizeof(FORM_DATA_STRING)-1) && 
                strncasecmp(ATTACHMENT_STRING, p, sizeof(ATTACHMENT_STRING)-1))
            return -1;

        filename_start = strstr(p, FILENAME_STRING);

        if(filename_start != 0) {
            char *q;
            
            filename_start += sizeof(FILENAME_STRING)-1;

            filename_end = filename_start + strcspn(filename_start, "\"");

            if(*filename_end != '\"')
                return -1;

            /*
             * IE sends full path, strip path from filename 
             * Also strip all UNIX path references
             */
            for(q = filename_end-1; q > filename_start; q--)
                if(*q == '\\' || *q == '/') {
                    filename_start = q+1;
                    break;
                }

            upload_ctx->file_name = (char*)ngx_pcalloc(upload_ctx->pool, filename_end - filename_start + 1);
            
            if(upload_ctx->file_name == 0)
                return -1; // Unable to allocate memory for string

            strncpy(upload_ctx->file_name, filename_start, filename_end - filename_start);
        }

        fieldname_start = p;

        do{
            fieldname_start = strstr(fieldname_start, FIELDNAME_STRING);
        }while((fieldname_start != 0) && (fieldname_start + sizeof(FIELDNAME_STRING) - 1 == filename_start));

        if(fieldname_start != 0) {
            fieldname_start += sizeof(FIELDNAME_STRING)-1;

            if(fieldname_start != filename_start) {
                fieldname_end = fieldname_start + strcspn(fieldname_start, "\"");

                if(*fieldname_end != '\"')
                    return -1;

                upload_ctx->field_name = (char*)ngx_pcalloc(upload_ctx->pool, fieldname_end - fieldname_start + 1);

                if(upload_ctx->field_name == 0)
                    return -1; // Unable to allocate memory for string

                strncpy(upload_ctx->field_name, fieldname_start, fieldname_end - fieldname_start);
            }
        }
    }else if(!strncasecmp(CONTENT_TYPE_STRING, header, sizeof(CONTENT_TYPE_STRING)-1)) {
        char *content_type_str = header + sizeof(CONTENT_TYPE_STRING)-1;
        size_t content_type_str_len; 
        
        content_type_str += strspn(content_type_str, " ");
        content_type_str_len = strlen(content_type_str);
        
        if(content_type_str_len == 0)
            return -1; // Empty Content-Type field

        upload_ctx->content_type = (char*)ngx_pcalloc(upload_ctx->pool, content_type_str_len+1);
        
        if(upload_ctx->content_type == 0)
            return -1; // Unable to allocate memory for string

        strncpy(upload_ctx->content_type, content_type_str, content_type_str_len);
    }
    return 0;
} /* }}} */

static void upload_free_file_attributes(ngx_http_upload_ctx_t *upload_ctx) { /* {{{ */
    upload_ctx->file_name = 0;
    upload_ctx->field_name = 0;
    upload_ctx->content_type = 0;
} /* }}} */

static int upload_start_file(ngx_http_upload_ctx_t *upload_ctx) { /* {{{ */
    // Call user-defined event handler
    if(upload_ctx->start_part_f)
        return upload_ctx->start_part_f(upload_ctx);
    else
        return 0;
} /* }}} */

static void upload_finish_file(ngx_http_upload_ctx_t *upload_ctx) { /* {{{ */
    // Call user-defined event handler
    if(upload_ctx->finish_part_f)
        upload_ctx->finish_part_f(upload_ctx);

    upload_free_file_attributes(upload_ctx);

    upload_ctx->discard_data = 0;
    upload_ctx->first_part = 0;
} /* }}} */

static void upload_abort_file(ngx_http_upload_ctx_t *upload_ctx) { /* {{{ */
    // Call user-defined event handler
    if(upload_ctx->abort_part_f)
        upload_ctx->abort_part_f(upload_ctx);

    upload_free_file_attributes(upload_ctx);

    upload_ctx->discard_data = 0;
    upload_ctx->first_part = 0;
} /* }}} */

static void upload_flush_output_buffer(ngx_http_upload_ctx_t *upload_ctx) { /* {{{ */
    if(upload_ctx->output_buffer_pos > upload_ctx->output_buffer) {
        if(upload_ctx->flush_output_buffer_f)
            if(upload_ctx->flush_output_buffer_f(upload_ctx, (void*)upload_ctx->output_buffer, 
                (size_t)(upload_ctx->output_buffer_pos - upload_ctx->output_buffer)) < 0)
                upload_ctx->discard_data = 1;

        upload_ctx->output_buffer_pos = upload_ctx->output_buffer;	
    }
} /* }}} */

void upload_init_ctx(ngx_http_upload_ctx_t *upload_ctx) { /* {{{ */
    upload_ctx->boundary_data = upload_ctx->boundary_start = upload_ctx->boundary_pos = 0;

	upload_ctx->upload_state = upload_state_boundary_seek;

    upload_ctx->header_accumulator_size = DEFAULT_HEADER_ACCUMULATOR_SIZE;

    upload_ctx->content_type = 0;
    upload_ctx->field_name = 0;
    upload_ctx->file_name = 0;

	upload_ctx->output_buffer_size = DEFAULT_OUTPUT_BUFFER_SIZE;

    upload_ctx->discard_data = 0;

	upload_ctx->start_part_f = ngx_http_upload_start;
	upload_ctx->finish_part_f = ngx_http_upload_finish;
	upload_ctx->abort_part_f = ngx_http_upload_abort;
	upload_ctx->flush_output_buffer_f = ngx_http_upload_flush_output_buffer;
} /* }}} */

void upload_shutdown_ctx(ngx_http_upload_ctx_t *upload_ctx) { /* {{{ */
	if(upload_ctx != 0) {
        // Abort file if we still processing it
        if(upload_ctx->upload_state == upload_state_data) {
            upload_flush_output_buffer(upload_ctx);
            upload_abort_file(upload_ctx);
        }

        upload_free_file_attributes(upload_ctx);
	}
} /* }}} */

int upload_start(ngx_http_upload_ctx_t *upload_ctx) { /* {{{ */
	if(upload_ctx == 0)
		return -1;

    // Allocate memory for header accumulator
	upload_ctx->header_accumulator = (char*)ngx_pcalloc(upload_ctx->pool, upload_ctx->header_accumulator_size);

	if(upload_ctx->header_accumulator == 0)
		return -1;

    // Allocate memory for output buffer
	upload_ctx->output_buffer = (u_char*)ngx_pcalloc(upload_ctx->pool, upload_ctx->output_buffer_size);

	if(upload_ctx->output_buffer == 0)
		return -1;

    upload_ctx->output_buffer_pos = upload_ctx->output_buffer;

    upload_reset_header_accumulator(upload_ctx);

    upload_ctx->first_part = 1;

	return 0;
} /* }}} */

int upload_parse_content_type(ngx_http_upload_ctx_t *upload_ctx, char *content_type) { /* {{{ */
    // Find comma in content type string, which terminates mime type
    char *mime_type_end_ptr = strchr(content_type, ';');
    char *boundary_start_ptr, *boundary_end_ptr;

    upload_ctx->boundary_data = 0;

    if(mime_type_end_ptr == 0)
        return -1; // No boundary found

    if(strncasecmp(MULTIPART_FORM_DATA_STRING, content_type, mime_type_end_ptr - content_type - 1))
        return -1; // Content is not a multipart/form-data 

    boundary_start_ptr = strstr(mime_type_end_ptr, BOUNDARY_STRING);

    if(boundary_start_ptr == 0)
        return -1; // No boundary found

    boundary_start_ptr += strlen(BOUNDARY_STRING);
    boundary_end_ptr = boundary_start_ptr + strcspn(boundary_start_ptr, " ;\n\r");

    if(boundary_end_ptr == boundary_start_ptr)
        return -1; // Boundary is empty

    // Allocate memory for entire boundary plus \r\n plus terminating character
    upload_ctx->boundary_data = (char*)ngx_pcalloc(upload_ctx->pool, boundary_end_ptr - boundary_start_ptr + 4 + 1);
    if(upload_ctx->boundary_data == 0)
        return -1;

    strncpy(upload_ctx->boundary_data + 4, boundary_start_ptr, boundary_end_ptr - boundary_start_ptr);
    
    upload_ctx->boundary_data[boundary_end_ptr - boundary_start_ptr + 4] = '\0';

    // Prepend boundary data by \r\n--
    upload_ctx->boundary_data[0] = '\r'; 
    upload_ctx->boundary_data[1] = '\n'; 
    upload_ctx->boundary_data[2] = '-'; 
    upload_ctx->boundary_data[3] = '-'; 

    /*
     * NOTE: first boundary doesn't start with \r\n. Here we
     * advance 2 positions forward. We will return 2 positions back 
     * later
     */
    upload_ctx->boundary_start = upload_ctx->boundary_data + 2;
    upload_ctx->boundary_pos  = upload_ctx->boundary_start;

    return 0;
} /* }}} */

void upload_set_output_buffer_size(ngx_http_upload_ctx_t *upload_ctx, size_t output_buffer_size) { /* {{{ */
    upload_ctx->output_buffer_size = output_buffer_size;
} /* }}} */

size_t upload_get_output_buffer_size(ngx_http_upload_ctx_t *upload_ctx) { /* {{{ */
    return upload_ctx->output_buffer_size;
} /* }}} */

void upload_set_header_accumulator_size(ngx_http_upload_ctx_t *upload_ctx, size_t header_accumulator_size) { /* {{{ */
    upload_ctx->header_accumulator_size = header_accumulator_size;
} /* }}} */

void upload_putc(ngx_http_upload_ctx_t *upload_ctx, u_char c) { /* {{{ */
    if(!upload_ctx->discard_data) {
        *upload_ctx->output_buffer_pos = c;

        upload_ctx->output_buffer_pos++;

        if(upload_ctx->output_buffer_pos == (upload_ctx->output_buffer + upload_ctx->output_buffer_size))
            upload_flush_output_buffer(upload_ctx);	
    }
} /* }}} */

/*
 * upload_process_buf
 *
 * Process buffer with multipart stream starting from start and terminating
 * by end, operating on upload_ctx. The headers are accumulated in upload context
 * and could be retrieved using upload_pop_header function.
 *
 * Returns value > 0 if context is ready to process next portion of data,
 *               = 0 if processing finished and remaining data could be discarded,
 *               -1 stream is malformed
 *               -2 insufficient memory 
 */
int upload_process_buf(ngx_http_upload_ctx_t *upload_ctx, u_char *start, u_char *end) { /* {{{ */

	u_char *p;

	// No more data?
	if(start == end) {
		if(upload_ctx->upload_state != upload_state_finish)
			return -1; // Signal error if still haven't finished
		else
			return 0; // Otherwise confirm end of stream
    }

	for(p = start; p != end; p++) {
		switch(upload_ctx->upload_state) {
			/*
			 * Seek the boundary
			 */
			case upload_state_boundary_seek:
				if(*p == *upload_ctx->boundary_pos) 
					upload_ctx->boundary_pos++;
				else
					upload_ctx->boundary_pos = upload_ctx->boundary_start;

				if(*upload_ctx->boundary_pos == '\0') {
					upload_ctx->upload_state = upload_state_after_boundary;
					upload_ctx->boundary_start = upload_ctx->boundary_data;
					upload_ctx->boundary_pos = upload_ctx->boundary_start;
				}
				break;
			case upload_state_after_boundary:
				switch(*p) {
					case '\n':
						upload_ctx->upload_state = upload_state_headers;
						upload_reset_header_accumulator(upload_ctx);
					case '\r':
						break;
					case '-':
						upload_ctx->upload_state = upload_state_finish;
						break;
				}
				break;
			/*
			 * Collect and store headers
			 */
			case upload_state_headers:
				switch(*p) {
					case '\n':
						if(strlen(upload_ctx->header_accumulator) == 0) {
                            upload_ctx->is_file = (upload_ctx->file_name == 0) || (*upload_ctx->file_name == '\0') ? 0 : 1;
                            
                            if(upload_start_file(upload_ctx) < 0) {
                                upload_ctx->upload_state = upload_state_finish;
                                return -1; // User requested to cancel processing
                            } else {
                                upload_ctx->upload_state = upload_state_data;
                                upload_ctx->output_buffer_pos = upload_ctx->output_buffer;	
                            }
                        } else {
                            if(upload_parse_part_header(upload_ctx, upload_ctx->header_accumulator) < 0) {
                                upload_ctx->upload_state = upload_state_finish;
                                return -1; // Malformed header
                            } else
                                upload_reset_header_accumulator(upload_ctx);
                        }
					case '\r':
						break;
					default:
                        // NOTE: Leaving space for final \0
						if(upload_ctx->header_accumulator_pos < upload_ctx->header_accumulator + upload_ctx->header_accumulator_size - 1)
							*upload_ctx->header_accumulator_pos++ = *p;
						else {
                            upload_ctx->upload_state = upload_state_finish;
							return -1; // Header is too long
                        }
						break;
				}
				break;
			/*
			 * Search for separating or terminating boundary
			 * and output data simultaneously
			 */
			case upload_state_data:
				if(*p == *upload_ctx->boundary_pos) 
					upload_ctx->boundary_pos++;
				else {
					if(upload_ctx->boundary_pos == upload_ctx->boundary_start) {
                        // IE 5.0 bug workaround
                        if(*p == '\n') {
                            /*
                             * Set current matched position beyond LF and prevent outputting
                             * CR in case of unsuccessful match by altering boundary_start 
                             */ 
                            upload_ctx->boundary_pos = upload_ctx->boundary_data + 2;
                            upload_ctx->boundary_start = upload_ctx->boundary_data + 1;
                        } else
                            upload_putc(upload_ctx, *p);
                    } else {
						// Output partially matched lump of boundary
						const char *q;
						for(q = upload_ctx->boundary_start; q != upload_ctx->boundary_pos; q++)
							upload_putc(upload_ctx, *q);

                        p--; // Repeat reading last character

						// And reset matched position
                        upload_ctx->boundary_start = upload_ctx->boundary_data;
						upload_ctx->boundary_pos = upload_ctx->boundary_start;
					}
				}

				if(*upload_ctx->boundary_pos == '\0') {
					upload_ctx->upload_state = upload_state_after_boundary;
					upload_ctx->boundary_pos = upload_ctx->boundary_start;

                    upload_flush_output_buffer(upload_ctx);
                    if(!upload_ctx->discard_data)
                        upload_finish_file(upload_ctx);
                    else
                        upload_abort_file(upload_ctx);
				}
				break;
			/*
			 * Skip trailing garbage
			 */
			case upload_state_finish:
				break;
		}
	}

	return (upload_ctx->upload_state == upload_state_finish) ? 0 : 1;
} /* }}} */

